/*
  Copyright (C) 2012  Philipp Kaelin

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * \author  Philipp Kaelin
 * \date    created:  2011/12/07 <br>
 *          modified: 2012/01/16
 * \brief   Finite State Machine for the main application
 * \defgroup Application Application Finite State Machine
 */

#ifndef TOASTER_H_
#define TOASTER_H_

#include <stdint.h>
#include "State.h"

/** Define the number of ticks in a second, generated by a timer interrupt */
#define TICKS_PER_SECOND  1

/** Macro to calculate the number of ticks in a second */
#define Seconds(x)        (x*TICKS_PER_SECOND)

/** Timeout in Seconds */
#define TIMEOUT           15

namespace Toaster {

  enum {
    /** Size of Event Queue */
    EventQueueSize = 4
  };

  /**
    * Define all the possible Events here
    */
  enum Event {
    // System Event
    NoEvent,
    Tick,
    Immediately,

    // User Event
    TasterStart,
    TasterFertig,

    // Hardware Event
    EndschalterDrinnen,
    EndschalterLeer,
  };

  struct Transition {
    State*      currentState;
    Event       event;
    uint16_t    maxTimeBeforeTransition;
    State*      nextState;
  };

  /**
   * \ingroup Application
   * \brief Initializes the event queue
   */
  void init();

  /**
   * \ingroup Application
   * \brief Send an Event to the Application
   * \note The Event must be processed by process()
   */
  void sendEvent(Event e);

  /**
   * \ingroup Application
   * \brief Process Event if one is available
   * \return True if there was an Event available
   */
  bool process();
}

#endif /* TOASTER_H_ */
